@@ -23,11 +23,13 @@
 
 const {
   Array,
+  ArrayBufferIsView,
   ArrayIsArray,
   ArrayPrototypeForEach,
   MathFloor,
   MathMin,
   MathTrunc,
+  NumberIsInteger,
   NumberIsNaN,
   NumberMAX_SAFE_INTEGER,
   NumberMIN_SAFE_INTEGER,
@@ -56,7 +58,7 @@ const {
   byteLengthUtf8,
   compare: _compare,
   compareOffset,
-  createFromString,
+  copy: _copy,
   fill: bindingFill,
   isAscii: bindingIsAscii,
   isUtf8: bindingIsUtf8,
@@ -147,11 +149,12 @@ const constants = ObjectDefineProperties({}, {
 });
 
 Buffer.poolSize = 8 * 1024;
-let poolSize, poolOffset, allocPool;
+let poolSize, poolOffset, allocPool, allocBuffer;
 
 function createPool() {
   poolSize = Buffer.poolSize;
-  allocPool = createUnsafeBuffer(poolSize).buffer;
+  allocBuffer = createUnsafeBuffer(poolSize);
+  allocPool = allocBuffer.buffer;
   markAsUntransferable(allocPool);
   poolOffset = 0;
 }
@@ -199,16 +202,16 @@ function toInteger(n, defaultVal) {
   return defaultVal;
 }
 
-function _copy(source, target, targetStart, sourceStart, sourceEnd) {
-  if (!isUint8Array(source))
+function copyImpl(source, target, targetStart, sourceStart, sourceEnd) {
+  if (!ArrayBufferIsView(source))
     throw new ERR_INVALID_ARG_TYPE('source', ['Buffer', 'Uint8Array'], source);
-  if (!isUint8Array(target))
+  if (!ArrayBufferIsView(target))
     throw new ERR_INVALID_ARG_TYPE('target', ['Buffer', 'Uint8Array'], target);
 
   if (targetStart === undefined) {
     targetStart = 0;
   } else {
-    targetStart = toInteger(targetStart, 0);
+    targetStart = NumberIsInteger(targetStart) ? targetStart : toInteger(targetStart, 0);
     if (targetStart < 0)
       throw new ERR_OUT_OF_RANGE('targetStart', '>= 0', targetStart);
   }
@@ -216,38 +219,38 @@ function _copy(source, target, targetStart, sourceStart, sourceEnd) {
   if (sourceStart === undefined) {
     sourceStart = 0;
   } else {
-    sourceStart = toInteger(sourceStart, 0);
-    if (sourceStart < 0 || sourceStart > source.length)
-      throw new ERR_OUT_OF_RANGE('sourceStart', `>= 0 && <= ${source.length}`, sourceStart);
+    sourceStart = NumberIsInteger(sourceStart) ? sourceStart : toInteger(sourceStart, 0);
+    if (sourceStart < 0 || sourceStart > source.byteLength)
+      throw new ERR_OUT_OF_RANGE('sourceStart', `>= 0 && <= ${source.byteLength}`, sourceStart);
   }
 
   if (sourceEnd === undefined) {
-    sourceEnd = source.length;
+    sourceEnd = source.byteLength;
   } else {
-    sourceEnd = toInteger(sourceEnd, 0);
+    sourceEnd = NumberIsInteger(sourceEnd) ? sourceEnd : toInteger(sourceEnd, 0);
     if (sourceEnd < 0)
       throw new ERR_OUT_OF_RANGE('sourceEnd', '>= 0', sourceEnd);
   }
 
-  if (targetStart >= target.length || sourceStart >= sourceEnd)
+  if (targetStart >= target.byteLength || sourceStart >= sourceEnd)
     return 0;
 
   return _copyActual(source, target, targetStart, sourceStart, sourceEnd);
 }
 
 function _copyActual(source, target, targetStart, sourceStart, sourceEnd) {
-  if (sourceEnd - sourceStart > target.length - targetStart)
-    sourceEnd = sourceStart + target.length - targetStart;
+  if (sourceEnd - sourceStart > target.byteLength - targetStart)
+    sourceEnd = sourceStart + target.byteLength - targetStart;
 
   let nb = sourceEnd - sourceStart;
-  const sourceLen = source.length - sourceStart;
+  const sourceLen = source.byteLength - sourceStart;
   if (nb > sourceLen)
     nb = sourceLen;
 
-  if (sourceStart !== 0 || sourceEnd < source.length)
-    source = new Uint8Array(source.buffer, source.byteOffset + sourceStart, nb);
+  if (nb <= 0)
+    return 0;
 
-  TypedArrayPrototypeSet(target, source, targetStart);
+  _copy(source, target, targetStart, sourceStart, nb);
 
   return nb;
 }
@@ -439,38 +442,49 @@ function allocate(size) {
 }
 
 function fromStringFast(string, ops) {
-  const length = ops.byteLength(string);
+  const maxLength = Buffer.poolSize >>> 1;
 
-  if (length >= (Buffer.poolSize >>> 1))
-    return createFromString(string, ops.encodingVal);
+  let length = string.length; // Min length
+
+  if (length >= maxLength)
+    return createFromString(string, ops);
+
+  length *= 4; // Max length (4 bytes per character)
+
+  if (length >= maxLength)
+    length = ops.byteLength(string); // Actual length
+
+  if (length >= maxLength)
+    return createFromString(string, ops, length);
 
   if (length > (poolSize - poolOffset))
     createPool();
-  let b = new FastBuffer(allocPool, poolOffset, length);
-  const actual = ops.write(b, string, 0, length);
-  if (actual !== length) {
-    // byteLength() may overestimate. That's a rare case, though.
-    b = new FastBuffer(allocPool, poolOffset, actual);
-  }
+
+  const actual = ops.write(allocBuffer, string, poolOffset, length);
+  const b = new FastBuffer(allocPool, poolOffset, actual);
+
   poolOffset += actual;
   alignPool();
   return b;
 }
 
+function createFromString(string, ops, length = ops.byteLength(string)) {
+  const buf = Buffer.allocUnsafeSlow(length);
+  const actual = ops.write(buf, string, 0, length);
+  return actual < length ? new FastBuffer(buf.buffer, 0, actual) : buf;
+}
+
 function fromString(string, encoding) {
   let ops;
-  if (typeof encoding !== 'string' || encoding.length === 0) {
-    if (string.length === 0)
-      return new FastBuffer();
+  if (!encoding || encoding === 'utf8') {
     ops = encodingOps.utf8;
   } else {
     ops = getEncodingOps(encoding);
     if (ops === undefined)
       throw new ERR_UNKNOWN_ENCODING(encoding);
-    if (string.length === 0)
-      return new FastBuffer();
   }
-  return fromStringFast(string, ops);
+
+  return string.length === 0 ? new FastBuffer() : fromStringFast(string, ops);
 }
 
 function fromArrayBuffer(obj, byteOffset, length) {
@@ -768,13 +782,22 @@ function byteLength(string, encoding) {
   if (len === 0)
     return 0;
 
-  if (encoding) {
-    const ops = getEncodingOps(encoding);
-    if (ops) {
-      return ops.byteLength(string);
-    }
+  if (!encoding || encoding === 'utf8') {
+    return byteLengthUtf8(string);
   }
-  return byteLengthUtf8(string);
+
+  if (encoding === 'ascii') {
+    return len;
+  }
+
+  const ops = getEncodingOps(encoding);
+  if (ops === undefined) {
+    // TODO (ronag): Makes more sense to throw here.
+    // throw new ERR_UNKNOWN_ENCODING(encoding);
+    return byteLengthUtf8(string);
+  }
+
+  return ops.byteLength(string);
 }
 
 Buffer.byteLength = byteLength;
@@ -801,7 +824,7 @@ ObjectDefineProperty(Buffer.prototype, 'offset', {
 
 Buffer.prototype.copy =
   function copy(target, targetStart, sourceStart, sourceEnd) {
-    return _copy(this, target, targetStart, sourceStart, sourceEnd);
+    return copyImpl(this, target, targetStart, sourceStart, sourceEnd);
   };
 
 // No need to verify that "buf.length <= MAX_UINT32" since it's a read-only
@@ -1105,8 +1128,10 @@ Buffer.prototype.write = function write(string, offset, length, encoding) {
     }
   }
 
-  if (!encoding)
+  if (!encoding || encoding === 'utf8')
     return this.utf8Write(string, offset, length);
+  if (encoding === 'ascii')
+    return this.asciiWrite(string, offset, length);
 
   const ops = getEncodingOps(encoding);
   if (ops === undefined)
