@@ -1314,23 +1314,23 @@ function mkdir(path, options, callback) {
   if (typeof options === 'function') {
     callback = options;
   } else if (typeof options === 'number' || typeof options === 'string') {
-    mode = options;
+    mode = parseFileMode(options, 'mode');
   } else if (options) {
-    if (options.recursive !== undefined)
+    if (options.recursive !== undefined) {
       recursive = options.recursive;
-    if (options.mode !== undefined)
-      mode = options.mode;
+      validateBoolean(recursive, 'options.recursive');
+    }
+    if (options.mode !== undefined) {
+      mode = parseFileMode(options.mode, 'options.mode');
+    }
   }
   callback = makeCallback(callback);
-  path = getValidatedPath(path);
-
-  validateBoolean(recursive, 'options.recursive');
 
   const req = new FSReqCallback();
   req.oncomplete = callback;
   binding.mkdir(
-    path,
-    parseFileMode(mode, 'mode'),
+    getValidatedPath(path),
+    mode,
     recursive,
     req,
   );
@@ -1349,19 +1349,20 @@ function mkdirSync(path, options) {
   let mode = 0o777;
   let recursive = false;
   if (typeof options === 'number' || typeof options === 'string') {
-    mode = options;
+    mode = parseFileMode(options, 'mode');
   } else if (options) {
-    if (options.recursive !== undefined)
+    if (options.recursive !== undefined) {
       recursive = options.recursive;
-    if (options.mode !== undefined)
-      mode = options.mode;
+      validateBoolean(recursive, 'options.recursive');
+    }
+    if (options.mode !== undefined) {
+      mode = parseFileMode(options.mode, 'options.mode');
+    }
   }
-  path = getValidatedPath(path);
-  validateBoolean(recursive, 'options.recursive');
 
   const result = binding.mkdir(
-    path,
-    parseFileMode(mode, 'mode'),
+    getValidatedPath(path),
+    mode,
     recursive,
   );
 
@@ -1549,7 +1550,8 @@ function lstat(path, options = { bigint: false }, callback) {
   callback = makeStatsCallback(callback);
   path = getValidatedPath(path);
   if (permission.isEnabled() && !permission.has('fs.read', path)) {
-    callback(new ERR_ACCESS_DENIED('Access to this API has been restricted', 'FileSystemRead', path));
+    const resource = BufferIsBuffer(path) ? BufferToString(path) : path;
+    callback(new ERR_ACCESS_DENIED('Access to this API has been restricted', 'FileSystemRead', resource));
     return;
   }
 
@@ -1626,7 +1628,8 @@ function fstatSync(fd, options = { bigint: false }) {
 function lstatSync(path, options = { bigint: false, throwIfNoEntry: true }) {
   path = getValidatedPath(path);
   if (permission.isEnabled() && !permission.has('fs.read', path)) {
-    throw new ERR_ACCESS_DENIED('Access to this API has been restricted', 'FileSystemRead', path);
+    const resource = BufferIsBuffer(path) ? BufferToString(path) : path;
+    throw new ERR_ACCESS_DENIED('Access to this API has been restricted', 'FileSystemRead', resource);
   }
   const stats = binding.lstat(
     getValidatedPath(path),
