@@ -182,6 +182,26 @@ const rawMethods = internalBinding('process_methods');
   process.kill = wrapped.kill;
   process.exit = wrapped.exit;
 
+  let finalizationMod;
+  ObjectDefineProperty(process, 'finalization', {
+    __proto__: null,
+    get() {
+      if (finalizationMod !== undefined) {
+        return finalizationMod;
+      }
+
+      const { createFinalization } = require('internal/process/finalization');
+      finalizationMod = createFinalization();
+
+      return finalizationMod;
+    },
+    set(value) {
+      finalizationMod = value;
+    },
+    enumerable: true,
+    configurable: true,
+  });
+
   process.hrtime = perThreadSetup.hrtime;
   process.hrtime.bigint = perThreadSetup.hrtimeBigInt;
 
