@@ -1175,15 +1175,8 @@ async function mkdtemp(prefix, options) {
   prefix = getValidatedPath(prefix, 'prefix');
   warnOnNonPortableTemplate(prefix);
 
-  let path;
-  if (typeof prefix === 'string') {
-    path = `${prefix}XXXXXX`;
-  } else {
-    path = Buffer.concat([prefix, Buffer.from('XXXXXX')]);
-  }
-
   return await PromisePrototypeThen(
-    binding.mkdtemp(path, options.encoding, kUsePromises),
+    binding.mkdtemp(prefix, options.encoding, kUsePromises),
     undefined,
     handleErrorFromBinding,
   );
