@@ -30,8 +30,6 @@ const { getOptionValue } = require('internal/options');
 const { sep, posix: { relative: relativePosixPath }, resolve } = require('path');
 const preserveSymlinks = getOptionValue('--preserve-symlinks');
 const preserveSymlinksMain = getOptionValue('--preserve-symlinks-main');
-const experimentalNetworkImports =
-  getOptionValue('--experimental-network-imports');
 const inputTypeFlag = getOptionValue('--input-type');
 const { URL, pathToFileURL, fileURLToPath, isURL, URLParse } = require('internal/url');
 const { getCWDURL, setOwnProperty } = require('internal/util');
@@ -48,7 +46,6 @@ const {
   ERR_PACKAGE_PATH_NOT_EXPORTED,
   ERR_UNSUPPORTED_DIR_IMPORT,
   ERR_UNSUPPORTED_RESOLVE_REQUEST,
-  ERR_NETWORK_IMPORT_DISALLOWED,
 } = require('internal/errors').codes;
 
 const { Module: CJSModule } = require('internal/modules/cjs/loader');
@@ -886,10 +883,6 @@ function moduleResolve(specifier, base, conditions, preserveSymlinks) {
     StringPrototypeSlice(base, 0, StringPrototypeIndexOf(base, ':') + 1) :
     base.protocol;
   const isData = protocol === 'data:';
-  const isRemote =
-    isData ||
-    protocol === 'http:' ||
-    protocol === 'https:';
   // Order swapped from spec for minor perf gain.
   // Ok since relative URLs cannot parse as URLs.
   let resolved;
@@ -907,7 +900,7 @@ function moduleResolve(specifier, base, conditions, preserveSymlinks) {
     try {
       resolved = new URL(specifier);
     } catch (cause) {
-      if (isRemote && !BuiltinModule.canBeRequiredWithoutScheme(specifier)) {
+      if (isData && !BuiltinModule.canBeRequiredWithoutScheme(specifier)) {
         const error = new ERR_UNSUPPORTED_RESOLVE_REQUEST(specifier, base);
         setOwnProperty(error, 'cause', cause);
         throw error;
@@ -976,57 +969,6 @@ function resolveAsCommonJS(specifier, parentURL) {
   }
 }
 
-/**
- * Throw an error if an import is not allowed.
- * TODO(@JakobJingleheimer): de-dupe `specifier` & `parsed`
- * @param {string} specifier - The import specifier.
- * @param {URL} parsed - The parsed URL of the import specifier.
- * @param {URL} parsedParentURL - The parsed URL of the parent module.
- * @throws {ERR_NETWORK_IMPORT_DISALLOWED} - If the import is disallowed.
- */
-function checkIfDisallowedImport(specifier, parsed, parsedParentURL) {
-  if (parsedParentURL) {
-    // Avoid accessing the `protocol` property due to the lazy getters.
-    const parentProtocol = parsedParentURL.protocol;
-    if (
-      parentProtocol === 'http:' ||
-      parentProtocol === 'https:'
-    ) {
-      if (shouldBeTreatedAsRelativeOrAbsolutePath(specifier)) {
-        // Avoid accessing the `protocol` property due to the lazy getters.
-        const parsedProtocol = parsed?.protocol;
-        // data: and blob: disallowed due to allowing file: access via
-        // indirection
-        if (parsedProtocol &&
-          parsedProtocol !== 'https:' &&
-          parsedProtocol !== 'http:'
-        ) {
-          throw new ERR_NETWORK_IMPORT_DISALLOWED(
-            specifier,
-            parsedParentURL,
-            'remote imports cannot import from a local location.',
-          );
-        }
-
-        return { url: parsed.href };
-      }
-      if (BuiltinModule.canBeRequiredWithoutScheme(specifier)) {
-        throw new ERR_NETWORK_IMPORT_DISALLOWED(
-          specifier,
-          parsedParentURL,
-          'remote imports cannot import from a local location.',
-        );
-      }
-
-      throw new ERR_NETWORK_IMPORT_DISALLOWED(
-        specifier,
-        parsedParentURL,
-        'only relative and absolute specifiers are supported.',
-      );
-    }
-  }
-}
-
 /**
  * Validate user-input in `context` supplied by a custom loader.
  * @param {string | URL | undefined} parentURL - The parent URL.
@@ -1068,39 +1010,11 @@ function defaultResolve(specifier, context = {}) {
     // Avoid accessing the `protocol` property due to the lazy getters.
     protocol = parsed.protocol;
 
-    if (protocol === 'data:' &&
-      parsedParentURL.protocol !== 'file:' &&
-      experimentalNetworkImports) {
-      throw new ERR_NETWORK_IMPORT_DISALLOWED(
-        specifier,
-        parsedParentURL,
-        'import data: from a non file: is not allowed',
-      );
-    }
-    if (protocol === 'data:' ||
-      (experimentalNetworkImports &&
-        (
-          protocol === 'https:' ||
-          protocol === 'http:'
-        )
-      )
-    ) {
+    if (protocol === 'data:') {
       return { __proto__: null, url: parsed.href };
     }
   }
 
-  // There are multiple deep branches that can either throw or return; instead
-  // of duplicating that deeply nested logic for the possible returns, DRY and
-  // check for a return. This seems the least gnarly.
-  const maybeReturn = checkIfDisallowedImport(
-    specifier,
-    parsed,
-    parsedParentURL,
-  );
-
-  if (maybeReturn) { return maybeReturn; }
-
-  // This must come after checkIfDisallowedImport
   protocol ??= parsed?.protocol;
   if (protocol === 'node:') { return { __proto__: null, url: specifier }; }
 
