@@ -81,6 +81,14 @@ function shouldUseESMLoader(mainPath) {
   if (mainPath && StringPrototypeEndsWith(mainPath, '.mjs')) { return true; }
   if (!mainPath || StringPrototypeEndsWith(mainPath, '.cjs')) { return false; }
 
+  if (getOptionValue('--experimental-strip-types')) {
+    // This ensures that --experimental-default-type=commonjs and .mts files are treated as commonjs
+    if (getOptionValue('--experimental-default-type') === 'commonjs') { return false; }
+    if (!mainPath || StringPrototypeEndsWith(mainPath, '.cts')) { return false; }
+    // This will likely change in the future to start with commonjs loader by default
+    if (mainPath && StringPrototypeEndsWith(mainPath, '.mts')) { return true; }
+  }
+
   const type = getNearestParentPackageJSONType(mainPath);
 
   // No package.json or no `type` field.
@@ -142,7 +150,7 @@ function runEntryPointWithESMLoader(callback) {
  * by `require('module')`) even when the entry point is ESM.
  * This monkey-patchable code is bypassed under `--experimental-default-type=module`.
  * Because of backwards compatibility, this function is exposed publicly via `import { runMain } from 'node:module'`.
- * When `--experimental-detect-module` is passed, this function will attempt to run ambiguous (no explicit extension, no
+ * Because of module detection, this function will attempt to run ambiguous (no explicit extension, no
  * `package.json` type field) entry points as CommonJS first; under certain conditions, it will retry running as ESM.
  * @param {string} main - First positional CLI argument, such as `'entry.js'` from `node entry.js`
  */
