@@ -36,6 +36,7 @@ const {
   exposeLazyInterfaces,
   defineReplaceableLazyAttribute,
   setupCoverageHooks,
+  emitExperimentalWarning,
 } = require('internal/util');
 
 const {
@@ -106,6 +107,7 @@ function prepareExecution(options) {
   const mainEntry = patchProcessObject(expandArgv1);
   setupTraceCategoryState();
   setupInspectorHooks();
+  setupNetworkInspection();
   setupNavigator();
   setupWarningHandler();
   setupUndici();
@@ -513,6 +515,16 @@ function setupInspectorHooks() {
   }
 }
 
+function setupNetworkInspection() {
+  if (internalBinding('config').hasInspector && getOptionValue('--experimental-network-inspection')) {
+    const {
+      enable,
+      disable,
+    } = require('internal/inspector_network_tracking');
+    internalBinding('inspector').setupNetworkTracking(enable, disable);
+  }
+}
+
 // In general deprecations are initialized wherever the APIs are implemented,
 // this is used to deprecate APIs implemented in C++ where the deprecation
 // utilities are not easily accessible.
@@ -613,8 +625,7 @@ function initializePermission() {
     };
     // Guarantee path module isn't monkey-patched to bypass permission model
     ObjectFreeze(require('path'));
-    process.emitWarning('Permission is an experimental feature',
-                        'ExperimentalWarning');
+    emitExperimentalWarning('Permission');
     const { has } = require('internal/process/permission');
     const warnFlags = [
       '--allow-addons',
@@ -703,8 +714,7 @@ function initializeSourceMapsHandlers() {
 
 function initializeFrozenIntrinsics() {
   if (getOptionValue('--frozen-intrinsics')) {
-    process.emitWarning('The --frozen-intrinsics flag is experimental',
-                        'ExperimentalWarning');
+    emitExperimentalWarning('Frozen intristics');
     require('internal/freeze_intrinsics')();
   }
 }
