@@ -25,7 +25,6 @@ const {
   readFileSync,
 } = require('fs');
 const { setupCoverageHooks } = require('internal/util');
-const { getOptionValue } = require('internal/options');
 const { tmpdir } = require('os');
 const { join, resolve, relative, matchesGlob } = require('path');
 const { fileURLToPath } = require('internal/url');
@@ -35,8 +34,6 @@ const kIgnoreRegex = /\/\* node:coverage ignore next (?<count>\d+ )?\*\//;
 const kLineEndingRegex = /\r?\n$/u;
 const kLineSplitRegex = /(?<=\r?\n)/u;
 const kStatusRegex = /\/\* node:coverage (?<status>enable|disable) \*\//;
-const excludeFileGlobs = getOptionValue('--test-coverage-exclude');
-const includeFileGlobs = getOptionValue('--test-coverage-include');
 
 class CoverageLine {
   constructor(line, startOffset, src, length = src?.length) {
@@ -53,10 +50,12 @@ class CoverageLine {
 }
 
 class TestCoverage {
-  constructor(coverageDirectory, originalCoverageDirectory, workingDirectory) {
+  constructor(coverageDirectory, originalCoverageDirectory, workingDirectory, excludeGlobs, includeGlobs) {
     this.coverageDirectory = coverageDirectory;
     this.originalCoverageDirectory = originalCoverageDirectory;
     this.workingDirectory = workingDirectory;
+    this.excludeGlobs = excludeGlobs;
+    this.includeGlobs = includeGlobs;
   }
 
   #sourceLines = new SafeMap();
@@ -311,7 +310,7 @@ class TestCoverage {
 
         const coverageFile = join(this.coverageDirectory, entry.name);
         const coverage = JSONParse(readFileSync(coverageFile, 'utf8'));
-        mergeCoverage(result, this.mapCoverageWithSourceMap(coverage), this.workingDirectory);
+        this.mergeCoverage(result, this.mapCoverageWithSourceMap(coverage));
       }
 
       return ArrayFrom(result.values());
@@ -334,7 +333,7 @@ class TestCoverage {
       const script = result[i];
       const { url, functions } = script;
 
-      if (shouldSkipFileCoverage(url, this.workingDirectory) || sourceMapCache[url] == null) {
+      if (this.shouldSkipFileCoverage(url) || sourceMapCache[url] == null) {
         newResult.set(url, script);
         continue;
       }
@@ -410,6 +409,54 @@ class TestCoverage {
     return MathMin(lines[line].startOffset + entry.originalColumn, lines[line].endOffset);
   }
 
+  mergeCoverage(merged, coverage) {
+    for (let i = 0; i < coverage.length; ++i) {
+      const newScript = coverage[i];
+      const { url } = newScript;
+
+      if (this.shouldSkipFileCoverage(url)) {
+        continue;
+      }
+
+      const oldScript = merged.get(url);
+
+      if (oldScript === undefined) {
+        merged.set(url, newScript);
+      } else {
+        mergeCoverageScripts(oldScript, newScript);
+      }
+    }
+  }
+
+  shouldSkipFileCoverage(url) {
+    // This check filters out core modules, which start with 'node:' in
+    // coverage reports, as well as any invalid coverages which have been
+    // observed on Windows.
+    if (!StringPrototypeStartsWith(url, 'file:')) return true;
+
+    const absolutePath = fileURLToPath(url);
+    const relativePath = relative(this.workingDirectory, absolutePath);
+
+    // This check filters out files that match the exclude globs.
+    if (this.excludeGlobs?.length > 0) {
+      for (let i = 0; i < this.excludeGlobs.length; ++i) {
+        if (matchesGlob(relativePath, this.excludeGlobs[i]) ||
+            matchesGlob(absolutePath, this.excludeGlobs[i])) return true;
+      }
+    }
+
+    // This check filters out files that do not match the include globs.
+    if (this.includeGlobs?.length > 0) {
+      for (let i = 0; i < this.includeGlobs.length; ++i) {
+        if (matchesGlob(relativePath, this.includeGlobs[i]) ||
+            matchesGlob(absolutePath, this.includeGlobs[i])) return false;
+      }
+      return true;
+    }
+
+    // This check filters out the node_modules/ directory, unless it is explicitly included.
+    return StringPrototypeIncludes(url, '/node_modules/');
+  }
 }
 
 function toPercentage(covered, total) {
@@ -420,7 +467,7 @@ function sortCoverageFiles(a, b) {
   return StringPrototypeLocaleCompare(a.path, b.path);
 }
 
-function setupCoverage() {
+function setupCoverage(options) {
   let originalCoverageDirectory = process.env.NODE_V8_COVERAGE;
   const cwd = process.cwd();
 
@@ -444,7 +491,13 @@ function setupCoverage() {
   // child processes.
   process.env.NODE_V8_COVERAGE = coverageDirectory;
 
-  return new TestCoverage(coverageDirectory, originalCoverageDirectory, cwd);
+  return new TestCoverage(
+    coverageDirectory,
+    originalCoverageDirectory,
+    cwd,
+    options.coverageExcludeGlobs,
+    options.coverageIncludeGlobs,
+  );
 }
 
 function mapRangeToLines(range, lines) {
@@ -488,55 +541,6 @@ function mapRangeToLines(range, lines) {
   return { __proto__: null, lines: mappedLines, ignoredLines };
 }
 
-function shouldSkipFileCoverage(url, workingDirectory) {
-  // This check filters out core modules, which start with 'node:' in
-  // coverage reports, as well as any invalid coverages which have been
-  // observed on Windows.
-  if (!StringPrototypeStartsWith(url, 'file:')) return true;
-
-  const absolutePath = fileURLToPath(url);
-  const relativePath = relative(workingDirectory, absolutePath);
-
-  // This check filters out files that match the exclude globs.
-  if (excludeFileGlobs?.length > 0) {
-    for (let i = 0; i < excludeFileGlobs.length; ++i) {
-      if (matchesGlob(relativePath, excludeFileGlobs[i]) ||
-          matchesGlob(absolutePath, excludeFileGlobs[i])) return true;
-    }
-  }
-
-  // This check filters out files that do not match the include globs.
-  if (includeFileGlobs?.length > 0) {
-    for (let i = 0; i < includeFileGlobs.length; ++i) {
-      if (matchesGlob(relativePath, includeFileGlobs[i]) ||
-          matchesGlob(absolutePath, includeFileGlobs[i])) return false;
-    }
-    return true;
-  }
-
-  // This check filters out the node_modules/ directory, unless it is explicitly included.
-  return StringPrototypeIncludes(url, '/node_modules/');
-}
-
-function mergeCoverage(merged, coverage, workingDirectory) {
-  for (let i = 0; i < coverage.length; ++i) {
-    const newScript = coverage[i];
-    const { url } = newScript;
-
-    if (shouldSkipFileCoverage(url, workingDirectory)) {
-      continue;
-    }
-
-    const oldScript = merged.get(url);
-
-    if (oldScript === undefined) {
-      merged.set(url, newScript);
-    } else {
-      mergeCoverageScripts(oldScript, newScript);
-    }
-  }
-}
-
 function mergeCoverageScripts(oldScript, newScript) {
   // Merge the functions from the new coverage into the functions from the
   // existing (merged) coverage.
