@@ -17,29 +17,62 @@ const {
   },
 } = require('internal/errors');
 const { exitCodes: { kGenericUserError } } = internalBinding('errors');
-
-const { kEmptyObject } = require('internal/util');
 const { kCancelledByParent, Test, Suite } = require('internal/test_runner/test');
 const {
   parseCommandLine,
   reporterScope,
-  setupTestReporters,
   shouldColorizeTestFiles,
 } = require('internal/test_runner/utils');
 const { queueMicrotask } = require('internal/process/task_queues');
 const { bigint: hrtime } = process.hrtime;
-
+const resolvedPromise = PromiseResolve();
 const testResources = new SafeMap();
+let globalRoot;
 
 testResources.set(reporterScope.asyncId(), reporterScope);
 
-function createTestTree(options = kEmptyObject) {
-  return setup(new Test({ __proto__: null, ...options, name: '<root>' }));
+function createTestTree(rootTestOptions, globalOptions) {
+  const harness = {
+    __proto__: null,
+    allowTestsToRun: false,
+    bootstrapPromise: resolvedPromise,
+    watching: false,
+    config: globalOptions,
+    coverage: null,
+    resetCounters() {
+      harness.counters = {
+        __proto__: null,
+        all: 0,
+        failed: 0,
+        passed: 0,
+        cancelled: 0,
+        skipped: 0,
+        todo: 0,
+        topLevel: 0,
+        suites: 0,
+      };
+    },
+    counters: null,
+    shouldColorizeTestFiles: shouldColorizeTestFiles(globalOptions.destinations),
+    teardown: null,
+    snapshotManager: null,
+  };
+
+  harness.resetCounters();
+  globalRoot = new Test({
+    __proto__: null,
+    ...rootTestOptions,
+    harness,
+    name: '<root>',
+  });
+  setupProcessState(globalRoot, globalOptions, harness);
+  globalRoot.startTime = hrtime();
+  return globalRoot;
 }
 
 function createProcessEventHandler(eventName, rootTest) {
   return (err) => {
-    if (!rootTest.harness.bootstrapComplete) {
+    if (rootTest.harness.bootstrapPromise) {
       // Something went wrong during the asynchronous portion of bootstrapping
       // the test runner. Since the test runner is not setup properly, we can't
       // do anything but throw the error.
@@ -95,7 +128,7 @@ function configureCoverage(rootTest, globalOptions) {
   const { setupCoverage } = require('internal/test_runner/coverage');
 
   try {
-    return setupCoverage();
+    return setupCoverage(globalOptions);
   } catch (err) {
     const msg = `Warning: Code coverage could not be enabled. ${err}`;
 
@@ -125,15 +158,7 @@ function collectCoverage(rootTest, coverage) {
   return summary;
 }
 
-function setup(root) {
-  if (root.startTime !== null) {
-    return root;
-  }
-
-  // Parse the command line options before the hook is enabled. We don't want
-  // global input validation errors to end up in the uncaughtException handler.
-  const globalOptions = parseCommandLine();
-
+function setupProcessState(root, globalOptions) {
   const hook = createHook({
     __proto__: null,
     init(asyncId, type, triggerAsyncId, resource) {
@@ -170,8 +195,13 @@ function setup(root) {
       kCancelledByParent));
 
     hook.disable();
-    process.removeListener('unhandledRejection', rejectionHandler);
     process.removeListener('uncaughtException', exceptionHandler);
+    process.removeListener('unhandledRejection', rejectionHandler);
+    process.removeListener('beforeExit', exitHandler);
+    if (globalOptions.isTestRunner) {
+      process.removeListener('SIGINT', terminationHandler);
+      process.removeListener('SIGTERM', terminationHandler);
+    }
   };
 
   const terminationHandler = () => {
@@ -188,64 +218,38 @@ function setup(root) {
     process.on('SIGTERM', terminationHandler);
   }
 
-  root.harness = {
-    __proto__: null,
-    allowTestsToRun: false,
-    bootstrapComplete: false,
-    watching: false,
-    coverage: FunctionPrototypeBind(collectCoverage, null, root, coverage),
-    resetCounters() {
-      root.harness.counters = {
-        __proto__: null,
-        all: 0,
-        failed: 0,
-        passed: 0,
-        cancelled: 0,
-        skipped: 0,
-        todo: 0,
-        topLevel: 0,
-        suites: 0,
-      };
-    },
-    counters: null,
-    shouldColorizeTestFiles: false,
-    teardown: exitHandler,
-    snapshotManager: null,
-  };
-  root.harness.resetCounters();
-  root.startTime = hrtime();
-  return root;
+  root.harness.coverage = FunctionPrototypeBind(collectCoverage, null, root, coverage);
+  root.harness.teardown = exitHandler;
 }
 
-let globalRoot;
-let reportersSetup;
-function getGlobalRoot() {
+function lazyBootstrapRoot() {
   if (!globalRoot) {
-    globalRoot = createTestTree();
+    // This is where the test runner is bootstrapped when node:test is used
+    // without the --test flag or the run() API.
+    const rootTestOptions = {
+      __proto__: null,
+      entryFile: process.argv?.[1],
+    };
+    const globalOptions = parseCommandLine();
+    createTestTree(rootTestOptions, globalOptions);
     globalRoot.reporter.on('test:fail', (data) => {
       if (data.todo === undefined || data.todo === false) {
         process.exitCode = kGenericUserError;
       }
     });
-    reportersSetup = setupTestReporters(globalRoot.reporter);
-    globalRoot.harness.shouldColorizeTestFiles ||= shouldColorizeTestFiles(globalRoot);
+    globalRoot.harness.bootstrapPromise = globalOptions.setup(globalRoot.reporter);
   }
   return globalRoot;
 }
 
-async function startSubtest(subtest) {
-  if (reportersSetup) {
+async function startSubtestAfterBootstrap(subtest) {
+  if (subtest.root.harness.bootstrapPromise) {
     // Only incur the overhead of awaiting the Promise once.
-    await reportersSetup;
-    reportersSetup = undefined;
-  }
-
-  const root = getGlobalRoot();
-  if (!root.harness.bootstrapComplete) {
-    root.harness.bootstrapComplete = true;
+    await subtest.root.harness.bootstrapPromise;
+    subtest.root.harness.bootstrapPromise = null;
     queueMicrotask(() => {
-      root.harness.allowTestsToRun = true;
-      root.processPendingSubtests();
+      subtest.root.harness.allowTestsToRun = true;
+      subtest.root.processPendingSubtests();
     });
   }
 
@@ -254,12 +258,13 @@ async function startSubtest(subtest) {
 
 function runInParentContext(Factory) {
   function run(name, options, fn, overrides) {
-    const parent = testResources.get(executionAsyncId()) || getGlobalRoot();
+    const parent = testResources.get(executionAsyncId()) || lazyBootstrapRoot();
     const subtest = parent.createSubtest(Factory, name, options, fn, overrides);
-    if (!(parent instanceof Suite)) {
-      return startSubtest(subtest);
+    if (parent instanceof Suite) {
+      return PromiseResolve();
     }
-    return PromiseResolve();
+
+    return startSubtestAfterBootstrap(subtest);
   }
 
   const test = (name, options, fn) => {
@@ -286,7 +291,7 @@ function runInParentContext(Factory) {
 
 function hook(hook) {
   return (fn, options) => {
-    const parent = testResources.get(executionAsyncId()) || getGlobalRoot();
+    const parent = testResources.get(executionAsyncId()) || lazyBootstrapRoot();
     parent.createHook(hook, fn, {
       __proto__: null,
       ...options,
