@@ -23,8 +23,8 @@ const {
 
 const {
   validateAbortSignal,
-  validateArray,
   validateNumber,
+  validateStringArray,
 } = require('internal/validators');
 
 const {
@@ -304,7 +304,7 @@ class MockTimers {
   }
 
   #clearTimer(timer) {
-    if (timer.priorityQueuePosition !== undefined) {
+    if (timer?.priorityQueuePosition !== undefined) {
       this.#executionQueue.removeAt(timer.priorityQueuePosition);
       timer.priorityQueuePosition = undefined;
     }
@@ -676,7 +676,7 @@ class MockTimers {
    */
   /**
    * Enables the MockTimers replacing the native timers with the fake ones.
-   * @param {EnableOptions} options
+   * @param {EnableOptions} [options]
    */
   enable(options = { __proto__: null, apis: SUPPORTED_APIS, now: 0 }) {
     const internalOptions = { __proto__: null, ...options };
@@ -696,7 +696,7 @@ class MockTimers {
       internalOptions.apis = SUPPORTED_APIS;
     }
 
-    validateArray(internalOptions.apis, 'options.apis');
+    validateStringArray(internalOptions.apis, 'options.apis');
     // Check that the timers passed are supported
     ArrayPrototypeForEach(internalOptions.apis, (timer) => {
       if (!ArrayPrototypeIncludes(SUPPORTED_APIS, timer)) {
