@@ -1,4 +1,5 @@
 'use strict';
+
 const {
   ArrayIsArray,
   ArrayPrototypeEvery,
@@ -10,7 +11,6 @@ const {
   ArrayPrototypeMap,
   ArrayPrototypePush,
   ArrayPrototypeShift,
-  ArrayPrototypeSlice,
   ArrayPrototypeSome,
   ArrayPrototypeSort,
   ObjectAssign,
@@ -32,8 +32,7 @@ const { spawn } = require('child_process');
 const { finished } = require('internal/streams/end-of-stream');
 const { resolve } = require('path');
 const { DefaultDeserializer, DefaultSerializer } = require('v8');
-// TODO(aduh95): switch to internal/readline/interface when backporting to Node.js 16.x is no longer a concern.
-const { createInterface } = require('readline');
+const { Interface } = require('internal/readline/interface');
 const { deserializeError } = require('internal/error_serdes');
 const { Buffer } = require('buffer');
 const { FilesWatcher } = require('internal/watch_mode/files_watcher');
@@ -70,7 +69,7 @@ const {
   convertStringToRegExp,
   countCompletedTest,
   kDefaultPattern,
-  shouldColorizeTestFiles,
+  parseCommandLine,
 } = require('internal/test_runner/utils');
 const { Glob } = require('internal/fs/glob');
 const { once } = require('events');
@@ -88,10 +87,12 @@ const kCanceledTests = new SafeSet()
 
 let kResistStopPropagation;
 
-function createTestFileList() {
+function createTestFileList(patterns) {
   const cwd = process.cwd();
-  const hasUserSuppliedPattern = process.argv.length > 1;
-  const patterns = hasUserSuppliedPattern ? ArrayPrototypeSlice(process.argv, 1) : [kDefaultPattern];
+  const hasUserSuppliedPattern = patterns != null;
+  if (!patterns || patterns.length === 0) {
+    patterns = [kDefaultPattern];
+  }
   const glob = new Glob(patterns, {
     __proto__: null,
     cwd,
@@ -345,7 +346,6 @@ function runTestFile(path, filesWatcher, opts) {
 
     let err;
 
-
     child.on('error', (error) => {
       err = error;
     });
@@ -354,7 +354,7 @@ function runTestFile(path, filesWatcher, opts) {
       subtest.parseMessage(data);
     });
 
-    const rl = createInterface({ __proto__: null, input: child.stderr });
+    const rl = new Interface({ __proto__: null, input: child.stderr });
     rl.on('line', (line) => {
       if (isInspectorMessage(line)) {
         process.stderr.write(line + '\n');
@@ -419,19 +419,24 @@ function watchFiles(testFiles, opts) {
   opts.root.harness.watching = true;
 
   watcher.on('changed', ({ owners, eventType }) => {
-    if (eventType === 'rename') {
-      const updatedTestFiles = createTestFileList();
+    if (!opts.hasFiles && eventType === 'rename') {
+      const updatedTestFiles = createTestFileList(opts.globPatterns);
 
       const newFileName = ArrayPrototypeFind(updatedTestFiles, (x) => !ArrayPrototypeIncludes(testFiles, x));
       const previousFileName = ArrayPrototypeFind(testFiles, (x) => !ArrayPrototypeIncludes(updatedTestFiles, x));
 
+      testFiles = updatedTestFiles;
+
       // When file renamed
       if (newFileName && previousFileName) {
         owners = new SafeSet().add(newFileName);
         watcher.filterFile(resolve(newFileName), owners);
       }
 
-      testFiles = updatedTestFiles;
+      if (!newFileName && previousFileName) {
+        return; // Avoid rerunning files when file deleted
+      }
+
     }
 
     watcher.unfilterFilesOwnedBy(owners);
@@ -483,6 +488,7 @@ function run(options = kEmptyObject) {
     watch,
     setup,
     only,
+    globPatterns,
   } = options;
 
   if (files != null) {
@@ -503,6 +509,16 @@ function run(options = kEmptyObject) {
   if (only != null) {
     validateBoolean(only, 'options.only');
   }
+  if (globPatterns != null) {
+    validateArray(globPatterns, 'options.globPatterns');
+  }
+
+  if (globPatterns?.length > 0 && files?.length > 0) {
+    throw new ERR_INVALID_ARG_VALUE(
+      'options.globPatterns', globPatterns, 'is not supported when specifying \'options.files\'',
+    );
+  }
+
   if (shard != null) {
     validateObject(shard, 'options.shard');
     // Avoid re-evaluating the shard object in case it's a getter
@@ -551,21 +567,29 @@ function run(options = kEmptyObject) {
     });
   }
 
-  const root = createTestTree({ __proto__: null, concurrency, timeout, signal });
-  root.harness.shouldColorizeTestFiles ||= shouldColorizeTestFiles(root);
+  const rootTestOptions = { __proto__: null, concurrency, timeout, signal };
+  const globalOptions = {
+    __proto__: null,
+    // parseCommandLine() should not be used here. However, The existing run()
+    // behavior has relied on it, so removing it must be done in a semver major.
+    ...parseCommandLine(),
+    setup,  // This line can be removed when parseCommandLine() is removed here.
+  };
+  const root = createTestTree(rootTestOptions, globalOptions);
 
   if (process.env.NODE_TEST_CONTEXT !== undefined) {
     process.emitWarning('node:test run() is being called recursively within a test file. skipping running files.');
     root.postRun();
     return root.reporter;
   }
-  let testFiles = files ?? createTestFileList();
+  let testFiles = files ?? createTestFileList(globPatterns);
 
   if (shard) {
     testFiles = ArrayPrototypeFilter(testFiles, (_, index) => index % shard.total === shard.index - 1);
   }
 
   let postRun = () => root.postRun();
+  let teardown = () => root.harness.teardown();
   let filesWatcher;
   const opts = {
     __proto__: null,
@@ -574,15 +598,18 @@ function run(options = kEmptyObject) {
     inspectPort,
     testNamePatterns,
     testSkipPatterns,
+    hasFiles: files != null,
+    globPatterns,
     only,
     forceExit,
   };
   if (watch) {
     filesWatcher = watchFiles(testFiles, opts);
     postRun = undefined;
+    teardown = undefined;
   }
   const runFiles = () => {
-    root.harness.bootstrapComplete = true;
+    root.harness.bootstrapPromise = null;
     root.harness.allowTestsToRun = true;
     return SafePromiseAllSettledReturnVoid(testFiles, (path) => {
       const subtest = runTestFile(path, filesWatcher, opts);
@@ -591,7 +618,8 @@ function run(options = kEmptyObject) {
     });
   };
 
-  PromisePrototypeThen(PromisePrototypeThen(PromiseResolve(setup?.(root.reporter)), runFiles), postRun);
+  const setupPromise = PromiseResolve(setup?.(root.reporter));
+  PromisePrototypeThen(PromisePrototypeThen(PromisePrototypeThen(setupPromise, runFiles), postRun), teardown);
 
   return root.reporter;
 }
