@@ -45,7 +45,6 @@ const {
   createDeferredCallback,
   countCompletedTest,
   isTestFailureError,
-  parseCommandLine,
 } = require('internal/test_runner/utils');
 const {
   createDeferredPromise,
@@ -75,19 +74,10 @@ const kHookFailure = 'hookFailed';
 const kDefaultTimeout = null;
 const noop = FunctionPrototype;
 const kShouldAbort = Symbol('kShouldAbort');
-const kFilename = process.argv?.[1];
 const kHookNames = ObjectSeal(['before', 'after', 'beforeEach', 'afterEach']);
 const kUnwrapErrors = new SafeSet()
   .add(kTestCodeFailure).add(kHookFailure)
   .add('uncaughtException').add('unhandledRejection');
-const {
-  forceExit,
-  sourceMaps,
-  testNamePatterns,
-  testSkipPatterns,
-  testOnlyFlag,
-  updateSnapshots,
-} = parseCommandLine();
 let kResistStopPropagation;
 let assertObj;
 let findSourceMap;
@@ -133,7 +123,7 @@ function lazyAssertObject(harness) {
     const { getOptionValue } = require('internal/options');
     if (getOptionValue('--experimental-test-snapshots')) {
       const { SnapshotManager } = require('internal/test_runner/snapshot');
-      harness.snapshotManager = new SnapshotManager(kFilename, updateSnapshots);
+      harness.snapshotManager = new SnapshotManager(harness.config.updateSnapshots);
       assertObj.set('snapshot', harness.snapshotManager.createAssert());
     }
   }
@@ -225,6 +215,10 @@ class TestContext {
     return this.#test.name;
   }
 
+  get filePath() {
+    return this.#test.entryFile;
+  }
+
   get fullName() {
     return getFullName(this.#test);
   }
@@ -308,23 +302,43 @@ class TestContext {
   }
 
   before(fn, options) {
-    this.#test
-      .createHook('before', fn, { __proto__: null, ...options, hookType: 'before', loc: getCallerLocation() });
+    this.#test.createHook('before', fn, {
+      __proto__: null,
+      ...options,
+      parent: this.#test,
+      hookType: 'before',
+      loc: getCallerLocation(),
+    });
   }
 
   after(fn, options) {
-    this.#test
-      .createHook('after', fn, { __proto__: null, ...options, hookType: 'after', loc: getCallerLocation() });
+    this.#test.createHook('after', fn, {
+      __proto__: null,
+      ...options,
+      parent: this.#test,
+      hookType: 'after',
+      loc: getCallerLocation(),
+    });
   }
 
   beforeEach(fn, options) {
-    this.#test
-      .createHook('beforeEach', fn, { __proto__: null, ...options, hookType: 'beforeEach', loc: getCallerLocation() });
+    this.#test.createHook('beforeEach', fn, {
+      __proto__: null,
+      ...options,
+      parent: this.#test,
+      hookType: 'beforeEach',
+      loc: getCallerLocation(),
+    });
   }
 
   afterEach(fn, options) {
-    this.#test
-      .createHook('afterEach', fn, { __proto__: null, ...options, hookType: 'afterEach', loc: getCallerLocation() });
+    this.#test.createHook('afterEach', fn, {
+      __proto__: null,
+      ...options,
+      parent: this.#test,
+      hookType: 'afterEach',
+      loc: getCallerLocation(),
+    });
   }
 }
 
@@ -343,6 +357,10 @@ class SuiteContext {
     return this.#suite.name;
   }
 
+  get filePath() {
+    return this.#suite.entryFile;
+  }
+
   get fullName() {
     return getFullName(this.#suite);
   }
@@ -357,7 +375,7 @@ class Test extends AsyncResource {
     super('Test');
 
     let { fn, name, parent } = options;
-    const { concurrency, loc, only, timeout, todo, skip, signal, plan } = options;
+    const { concurrency, entryFile, loc, only, timeout, todo, skip, signal, plan } = options;
 
     if (typeof fn !== 'function') {
       fn = noop;
@@ -379,14 +397,17 @@ class Test extends AsyncResource {
     this.filtered = false;
 
     if (parent === null) {
+      this.root = this;
+      this.harness = options.harness;
+      this.config = this.harness.config;
       this.concurrency = 1;
       this.nesting = 0;
-      this.only = testOnlyFlag;
+      this.only = this.config.only;
       this.reporter = new TestsStream();
       this.runOnlySubtests = this.only;
       this.childNumber = 0;
       this.timeout = kDefaultTimeout;
-      this.root = this;
+      this.entryFile = entryFile;
       this.hooks = {
         __proto__: null,
         before: [],
@@ -399,14 +420,17 @@ class Test extends AsyncResource {
       const nesting = parent.parent === null ? parent.nesting :
         parent.nesting + 1;
 
+      this.root = parent.root;
+      this.harness = null;
+      this.config = this.root.harness.config;
       this.concurrency = parent.concurrency;
       this.nesting = nesting;
-      this.only = only ?? !parent.runOnlySubtests;
+      this.only = only ?? (parent.only && !parent.runOnlySubtests);
       this.reporter = parent.reporter;
-      this.runOnlySubtests = !this.only;
+      this.runOnlySubtests = false;
       this.childNumber = parent.subtests.length + 1;
       this.timeout = parent.timeout;
-      this.root = parent.root;
+      this.entryFile = parent.entryFile;
       this.hooks = {
         __proto__: null,
         before: [],
@@ -421,7 +445,7 @@ class Test extends AsyncResource {
         this.parent.filteredSubtestCount++;
       }
 
-      if (testOnlyFlag && only === false) {
+      if (this.config.only && only === false) {
         fn = noop;
       }
     }
@@ -471,7 +495,6 @@ class Test extends AsyncResource {
     );
 
     this.fn = fn;
-    this.harness = null; // Configured on the root test by the test harness.
     this.mock = null;
     this.plan = null;
     this.expectedAssertions = plan;
@@ -492,13 +515,13 @@ class Test extends AsyncResource {
     this.waitingOn = 0;
     this.finished = false;
 
-    if (!testOnlyFlag && (only || this.runOnlySubtests)) {
+    if (!this.config.only && (only || this.parent?.runOnlySubtests)) {
       const warning =
         "'only' and 'runOnly' require the --test-only command-line option.";
       this.diagnostic(warning);
     }
 
-    if (loc === undefined || kFilename === undefined) {
+    if (loc === undefined) {
       this.loc = undefined;
     } else {
       this.loc = {
@@ -508,7 +531,7 @@ class Test extends AsyncResource {
         file: loc[2],
       };
 
-      if (sourceMaps === true) {
+      if (this.config.sourceMaps === true) {
         const map = lazyFindSourceMap(this.loc.file);
         const entry = map?.findEntry(this.loc.line - 1, this.loc.column - 1);
 
@@ -526,7 +549,9 @@ class Test extends AsyncResource {
   }
 
   willBeFiltered() {
-    if (testOnlyFlag && !this.only) return true;
+    if (this.config.only && !this.only) return true;
+
+    const { testNamePatterns, testSkipPatterns } = this.config;
 
     if (testNamePatterns && !testMatchesPattern(this, testNamePatterns)) {
       return true;
@@ -754,10 +779,7 @@ class Test extends AsyncResource {
   }
 
   [kShouldAbort]() {
-    if (this.signal.aborted) {
-      return true;
-    }
-    if (this.outerSignal?.aborted) {
+    if (this.signal.aborted || this.outerSignal?.aborted) {
       this.#abortHandler();
       return true;
     }
@@ -857,10 +879,7 @@ class Test extends AsyncResource {
         await SafePromiseRace([PromiseResolve(promise), stopPromise]);
       }
 
-      if (this[kShouldAbort]()) {
-        this.postRun();
-        return;
-      }
+      this[kShouldAbort]();
       this.plan?.check();
       this.pass();
       await afterEach();
@@ -896,7 +915,7 @@ class Test extends AsyncResource {
       // This helps catch any asynchronous activity that occurs after the tests
       // have finished executing.
       this.postRun();
-    } else if (forceExit) {
+    } else if (this.config.forceExit) {
       // This is the root test, and all known tests and hooks have finished
       // executing. If the user wants to force exit the process regardless of
       // any remaining ref'ed handles, then do that now. It is theoretically
@@ -967,7 +986,7 @@ class Test extends AsyncResource {
 
       // Call this harness.coverage() before collecting diagnostics, since failure to collect coverage is a diagnostic.
       const coverage = harness.coverage();
-      harness.snapshotManager?.writeSnapshotFile();
+      harness.snapshotManager?.writeSnapshotFiles();
       for (let i = 0; i < diagnostics.length; i++) {
         reporter.diagnostic(nesting, loc, diagnostics[i]);
       }
@@ -1085,14 +1104,17 @@ class Test extends AsyncResource {
 class TestHook extends Test {
   #args;
   constructor(fn, options) {
-    if (options === null || typeof options !== 'object') {
-      options = kEmptyObject;
-    }
-    const { loc, timeout, signal } = options;
-    super({ __proto__: null, fn, loc, timeout, signal });
-
-    this.parentTest = options.parent ?? null;
-    this.hookType = options.hookType;
+    const { hookType, loc, parent, timeout, signal } = options;
+    super({
+      __proto__: null,
+      fn,
+      loc,
+      timeout,
+      signal,
+      harness: parent.root.harness,
+    });
+    this.parentTest = parent;
+    this.hookType = hookType;
   }
   run(args) {
     if (this.error && !this.outerSignal?.aborted) {
@@ -1116,9 +1138,7 @@ class TestHook extends Test {
     const { error, loc, parentTest: parent } = this;
 
     // Report failures in the root test's after() hook.
-    if (error && parent !== null &&
-        parent === parent.root && this.hookType === 'after') {
-
+    if (error && parent === parent.root && this.hookType === 'after') {
       if (isTestFailureError(error)) {
         error.failureType = kHookFailure;
       }
@@ -1138,11 +1158,13 @@ class Suite extends Test {
   constructor(options) {
     super(options);
 
-    if (testNamePatterns !== null && testSkipPatterns !== null && !options.skip) {
+    if (this.config.testNamePatterns !== null &&
+        this.config.testSkipPatterns !== null &&
+        !options.skip) {
       this.fn = options.fn || this.fn;
       this.skipped = false;
     }
-    this.runOnlySubtests = testOnlyFlag;
+    this.runOnlySubtests = this.config.only;
 
     try {
       const { ctx, args } = this.getRunArgs();
@@ -1159,23 +1181,23 @@ class Suite extends Test {
       );
     } catch (err) {
       this.fail(new ERR_TEST_FAILURE(err, kTestCodeFailure));
-
-      this.buildPhaseFinished = true;
+      this.postBuild();
     }
     this.fn = noop;
   }
 
   postBuild() {
     this.buildPhaseFinished = true;
-    if (this.filtered && this.filteredSubtestCount !== this.subtests.length) {
+    if (this.filtered &&
+        (this.filteredSubtestCount !== this.subtests.length || this.error)) {
       // A suite can transition from filtered to unfiltered based on the
       // tests that it contains - in case of children matching patterns.
       this.filtered = false;
       this.parent.filteredSubtestCount--;
     } else if (
-      testOnlyFlag &&
-      testNamePatterns == null &&
-      testSkipPatterns == null &&
+      this.config.only &&
+      this.config.testNamePatterns == null &&
+      this.config.testSkipPatterns == null &&
       this.filteredSubtestCount === this.subtests.length
     ) {
       // If no subtests are marked as "only", run them all
