@@ -1,24 +1,26 @@
 'use strict';
 const {
+  ArrayPrototypeFlatMap,
   ArrayPrototypeJoin,
   ArrayPrototypeMap,
-  ArrayPrototypeFlatMap,
   ArrayPrototypePush,
   ArrayPrototypeReduce,
   ArrayPrototypeSome,
-  ObjectGetOwnPropertyDescriptor,
   MathFloor,
   MathMax,
   MathMin,
+  NumberParseInt,
   NumberPrototypeToFixed,
-  SafePromiseAllReturnArrayLike,
+  ObjectGetOwnPropertyDescriptor,
   RegExp,
   RegExpPrototypeExec,
   SafeMap,
-  StringPrototypePadStart,
+  SafePromiseAllReturnArrayLike,
   StringPrototypePadEnd,
+  StringPrototypePadStart,
   StringPrototypeRepeat,
   StringPrototypeSlice,
+  StringPrototypeSplit,
 } = primordials;
 
 const { AsyncResource } = require('async_hooks');
@@ -129,10 +131,9 @@ function tryBuiltinReporter(name) {
   return require(builtinPath);
 }
 
-function shouldColorizeTestFiles(rootTest) {
+function shouldColorizeTestFiles(destinations) {
   // This function assumes only built-in destinations (stdout/stderr) supports coloring
-  const { reporters, destinations } = parseCommandLine();
-  return ArrayPrototypeSome(reporters, (_, index) => {
+  return ArrayPrototypeSome(destinations, (_, index) => {
     const destination = kBuiltinDestinations.get(destinations[index]);
     return destination && shouldColorize(destination);
   });
@@ -154,8 +155,8 @@ async function getReportersMap(reporters, destinations) {
         parentURL = 'file:///';
       }
 
-      const { esmLoader } = require('internal/process/esm_loader');
-      reporter = await esmLoader.import(name, parentURL, { __proto__: null });
+      const cascadedLoader = require('internal/modules/esm/loader').getOrInitializeCascadedLoader();
+      reporter = await cascadedLoader.import(name, parentURL, { __proto__: null });
     }
 
     if (reporter?.default) {
@@ -175,15 +176,6 @@ async function getReportersMap(reporters, destinations) {
 }
 
 const reporterScope = new AsyncResource('TestReporterScope');
-const setupTestReporters = reporterScope.bind(async (rootReporter) => {
-  const { reporters, destinations } = parseCommandLine();
-  const reportersMap = await getReportersMap(reporters, destinations);
-  for (let i = 0; i < reportersMap.length; i++) {
-    const { reporter, destination } = reportersMap[i];
-    compose(rootReporter, reporter).pipe(destination);
-  }
-});
-
 let globalTestOptions;
 
 function parseCommandLine() {
@@ -193,12 +185,22 @@ function parseCommandLine() {
 
   const isTestRunner = getOptionValue('--test');
   const coverage = getOptionValue('--experimental-test-coverage');
+  const forceExit = getOptionValue('--test-force-exit');
+  const sourceMaps = getOptionValue('--enable-source-maps');
+  const updateSnapshots = getOptionValue('--test-update-snapshots');
+  const watch = getOptionValue('--watch');
   const isChildProcess = process.env.NODE_TEST_CONTEXT === 'child';
   const isChildProcessV8 = process.env.NODE_TEST_CONTEXT === 'child-v8';
+  let concurrency;
+  let coverageExcludeGlobs;
+  let coverageIncludeGlobs;
   let destinations;
+  let only;
   let reporters;
+  let shard;
   let testNamePatterns;
-  let testOnlyFlag;
+  let testSkipPatterns;
+  let timeout;
 
   if (isChildProcessV8) {
     kBuiltinReporters.set('v8-serializer', 'internal/test_runner/reporter/v8-serializer');
@@ -228,26 +230,76 @@ function parseCommandLine() {
   }
 
   if (isTestRunner) {
-    testOnlyFlag = false;
+    timeout = getOptionValue('--test-timeout') || Infinity;
+    concurrency = getOptionValue('--test-concurrency') || true;
+    only = false;
     testNamePatterns = null;
+
+    const shardOption = getOptionValue('--test-shard');
+    if (shardOption) {
+      if (!RegExpPrototypeExec(/^\d+\/\d+$/, shardOption)) {
+        throw new ERR_INVALID_ARG_VALUE(
+          '--test-shard',
+          shardOption,
+          'must be in the form of <index>/<total>',
+        );
+      }
+
+      const indexAndTotal = StringPrototypeSplit(shardOption, '/');
+      shard = {
+        __proto__: null,
+        index: NumberParseInt(indexAndTotal[0], 10),
+        total: NumberParseInt(indexAndTotal[1], 10),
+      };
+    }
   } else {
+    timeout = Infinity;
+    concurrency = 1;
     const testNamePatternFlag = getOptionValue('--test-name-pattern');
-    testOnlyFlag = getOptionValue('--test-only');
+    only = getOptionValue('--test-only');
     testNamePatterns = testNamePatternFlag?.length > 0 ?
       ArrayPrototypeMap(
         testNamePatternFlag,
         (re) => convertStringToRegExp(re, '--test-name-pattern'),
       ) : null;
+    const testSkipPatternFlag = getOptionValue('--test-skip-pattern');
+    testSkipPatterns = testSkipPatternFlag?.length > 0 ?
+      ArrayPrototypeMap(testSkipPatternFlag, (re) => convertStringToRegExp(re, '--test-skip-pattern')) : null;
   }
 
+  if (coverage) {
+    coverageExcludeGlobs = getOptionValue('--test-coverage-exclude');
+    coverageIncludeGlobs = getOptionValue('--test-coverage-include');
+  }
+
+  const setup = reporterScope.bind(async (rootReporter) => {
+    const reportersMap = await getReportersMap(reporters, destinations);
+
+    for (let i = 0; i < reportersMap.length; i++) {
+      const { reporter, destination } = reportersMap[i];
+      compose(rootReporter, reporter).pipe(destination);
+    }
+  });
+
   globalTestOptions = {
     __proto__: null,
     isTestRunner,
+    concurrency,
     coverage,
-    testOnlyFlag,
-    testNamePatterns,
-    reporters,
+    coverageExcludeGlobs,
+    coverageIncludeGlobs,
     destinations,
+    forceExit,
+    only,
+    reporters,
+    setup,
+    shard,
+    sourceMaps,
+    testNamePatterns,
+    testSkipPatterns,
+    timeout,
+    updateSnapshots,
+    watch,
   };
 
   return globalTestOptions;
@@ -429,7 +481,6 @@ module.exports = {
   kDefaultPattern,
   parseCommandLine,
   reporterScope,
-  setupTestReporters,
   shouldColorizeTestFiles,
   getCoverageReport,
 };
