@@ -37,6 +37,7 @@ const {
   SafeWeakRef,
   StringPrototypeIncludes,
   StringPrototypeReplace,
+  StringPrototypeStartsWith,
   StringPrototypeToLowerCase,
   StringPrototypeToUpperCase,
   Symbol,
@@ -70,6 +71,9 @@ const { encodings } = internalBinding('string_decoder');
 
 const noCrypto = !process.versions.openssl;
 
+const isWindows = process.platform === 'win32';
+const isMacOS = process.platform === 'darwin';
+
 const experimentalWarnings = new SafeSet();
 
 const colorRegExp = /\u001b\[\d\d?m/g; // eslint-disable-line no-control-regex
@@ -515,11 +519,14 @@ function isInsideNodeModules() {
   if (ArrayIsArray(stack)) {
     for (const frame of stack) {
       const filename = frame.getFileName();
-      // If a filename does not start with / or contain \,
-      // it's likely from Node.js core.
+
       if (
-        filename[0] !== '/' &&
-        StringPrototypeIncludes(filename, '\\') === false
+        filename == null ||
+        StringPrototypeStartsWith(filename, 'node:') === true ||
+        (
+          filename[0] !== '/' &&
+          StringPrototypeIncludes(filename, '\\') === false
+        )
       ) {
         continue;
       }
@@ -913,6 +920,8 @@ module.exports = {
   isArrayBufferDetached,
   isError,
   isInsideNodeModules,
+  isMacOS,
+  isWindows,
   join,
   lazyDOMException,
   lazyDOMExceptionClass,
