@@ -459,6 +459,7 @@ function newReadableStreamFromStreamReadable(streamReadable, options = kEmptyObj
   const strategy = evaluateStrategyOrFallback(options?.strategy);
 
   let controller;
+  let wasCanceled = false;
 
   function onData(chunk) {
     // Copy the Buffer to detach it from the pool.
@@ -480,6 +481,10 @@ function newReadableStreamFromStreamReadable(streamReadable, options = kEmptyObj
     streamReadable.on('error', () => {});
     if (error)
       return controller.error(error);
+    // Was already canceled
+    if (wasCanceled) {
+      return;
+    }
     controller.close();
   });
 
@@ -491,6 +496,7 @@ function newReadableStreamFromStreamReadable(streamReadable, options = kEmptyObj
     pull() { streamReadable.resume(); },
 
     cancel(reason) {
+      wasCanceled = true;
       destroy(streamReadable, reason);
     },
   }, strategy);
