@@ -22,12 +22,16 @@
 'use strict';
 
 const {
+  ArrayPrototypeJoin,
+  ArrayPrototypeSlice,
   FunctionPrototypeBind,
   StringPrototypeCharCodeAt,
   StringPrototypeIndexOf,
   StringPrototypeLastIndexOf,
+  StringPrototypeRepeat,
   StringPrototypeReplace,
   StringPrototypeSlice,
+  StringPrototypeSplit,
   StringPrototypeToLowerCase,
 } = primordials;
 
@@ -539,6 +543,42 @@ const win32 = {
     if (from === to)
       return '';
 
+    if (fromOrig.length !== from.length || toOrig.length !== to.length) {
+      const fromSplit = StringPrototypeSplit(fromOrig, '\\');
+      const toSplit = StringPrototypeSplit(toOrig, '\\');
+      if (fromSplit[fromSplit.length - 1] === '') {
+        fromSplit.pop();
+      }
+      if (toSplit[toSplit.length - 1] === '') {
+        toSplit.pop();
+      }
+
+      const fromLen = fromSplit.length;
+      const toLen = toSplit.length;
+      const length = fromLen < toLen ? fromLen : toLen;
+
+      let i;
+      for (i = 0; i < length; i++) {
+        if (StringPrototypeToLowerCase(fromSplit[i]) !== StringPrototypeToLowerCase(toSplit[i])) {
+          break;
+        }
+      }
+
+      if (i === 0) {
+        return toOrig;
+      } else if (i === length) {
+        if (toLen > length) {
+          return ArrayPrototypeJoin(ArrayPrototypeSlice(toSplit, i), '\\');
+        }
+        if (fromLen > length) {
+          return StringPrototypeRepeat('..\\', fromLen - 1 - i) + '..';
+        }
+        return '';
+      }
+
+      return StringPrototypeRepeat('..\\', fromLen - i) + ArrayPrototypeJoin(ArrayPrototypeSlice(toSplit, i), '\\');
+    }
+
     // Trim any leading backslashes
     let fromStart = 0;
     while (fromStart < from.length &&
@@ -775,7 +815,7 @@ const win32 = {
    */
   basename(path, suffix) {
     if (suffix !== undefined)
-      validateString(suffix, 'ext');
+      validateString(suffix, 'suffix');
     validateString(path, 'path');
     let start = 0;
     let end = -1;
@@ -1196,20 +1236,20 @@ const posix = {
   join(...args) {
     if (args.length === 0)
       return '.';
-    let joined;
+
+    const path = [];
     for (let i = 0; i < args.length; ++i) {
       const arg = args[i];
       validateString(arg, 'path');
       if (arg.length > 0) {
-        if (joined === undefined)
-          joined = arg;
-        else
-          joined += `/${arg}`;
+        path.push(arg);
       }
     }
-    if (joined === undefined)
+
+    if (path.length === 0)
       return '.';
-    return posix.normalize(joined);
+
+    return posix.normalize(ArrayPrototypeJoin(path, '/'));
   },
 
   /**
@@ -1335,7 +1375,7 @@ const posix = {
    */
   basename(path, suffix) {
     if (suffix !== undefined)
-      validateString(suffix, 'ext');
+      validateString(suffix, 'suffix');
     validateString(path, 'path');
 
     let start = 0;
