@@ -4,6 +4,7 @@
 #include "node_errors.h"
 #include "node_external_reference.h"
 #include "node_url.h"
+#include "path.h"
 #include "permission/permission.h"
 #include "permission/permission_base.h"
 #include "util-inl.h"
@@ -241,7 +242,7 @@ void BindingData::ReadPackageJSON(const FunctionCallbackInfo<Value>& args) {
   Realm* realm = Realm::GetCurrent(args);
   auto isolate = realm->isolate();
 
-  Utf8Value path(isolate, args[0]);
+  BufferValue path(isolate, args[0]);
   bool is_esm = args[1]->IsTrue();
   auto error_context = ErrorContext();
   if (is_esm) {
@@ -261,15 +262,10 @@ void BindingData::ReadPackageJSON(const FunctionCallbackInfo<Value>& args) {
       permission::PermissionScope::kFileSystemRead,
       path.ToStringView());
 
-  // TODO(StefanStojanovic): Remove ifdef after
-  // path.toNamespacedPath logic is ported to C++
-#ifdef _WIN32
+  ToNamespacedPath(realm->env(), &path);
   auto package_json = GetPackageJSON(
-      realm, "\\\\?\\" + path.ToString(), is_esm ? &error_context : nullptr);
-#else
-  auto package_json =
-      GetPackageJSON(realm, path.ToString(), is_esm ? &error_context : nullptr);
-#endif
+      realm, path.ToStringView(), is_esm ? &error_context : nullptr);
+
   if (package_json == nullptr) {
     return;
   }
@@ -323,9 +319,21 @@ void BindingData::GetNearestParentPackageJSON(
   CHECK(args[0]->IsString());
 
   Realm* realm = Realm::GetCurrent(args);
-  Utf8Value path_value(realm->isolate(), args[0]);
+  BufferValue path_value(realm->isolate(), args[0]);
+  // Check if the path has a trailing slash. If so, add it after
+  // ToNamespacedPath() as it will be deleted by ToNamespacedPath()
+  bool slashCheck = path_value.ToStringView().ends_with(
+      std::filesystem::path::preferred_separator);
+
+  ToNamespacedPath(realm->env(), &path_value);
+
+  std::string path_value_str = path_value.ToString();
+  if (slashCheck) {
+    path_value_str.push_back(std::filesystem::path::preferred_separator);
+  }
+
   auto package_json =
-      TraverseParent(realm, std::filesystem::path(path_value.ToString()));
+      TraverseParent(realm, std::filesystem::path(path_value_str));
 
   if (package_json != nullptr) {
     args.GetReturnValue().Set(package_json->Serialize(realm));
@@ -338,9 +346,21 @@ void BindingData::GetNearestParentPackageJSONType(
   CHECK(args[0]->IsString());
 
   Realm* realm = Realm::GetCurrent(args);
-  Utf8Value path(realm->isolate(), args[0]);
+  BufferValue path_value(realm->isolate(), args[0]);
+  // Check if the path has a trailing slash. If so, add it after
+  // ToNamespacedPath() as it will be deleted by ToNamespacedPath()
+  bool slashCheck = path_value.ToStringView().ends_with(
+      std::filesystem::path::preferred_separator);
+
+  ToNamespacedPath(realm->env(), &path_value);
+
+  std::string path_value_str = path_value.ToString();
+  if (slashCheck) {
+    path_value_str.push_back(std::filesystem::path::preferred_separator);
+  }
+
   auto package_json =
-      TraverseParent(realm, std::filesystem::path(path.ToString()));
+      TraverseParent(realm, std::filesystem::path(path_value_str));
 
   if (package_json == nullptr) {
     return;
